
#ifndef DOSDRVR_H_INCLUDED
#define DOSDRVR_H_INCLUDED

typedef struct DeviceHeader_s 
		{
		struct DeviceHeader_s _far *next;
		unsigned short Flags;
		unsigned short StrategyOffset;
		unsigned short InterruptOffset;
		unsigned char Name[8];
		} DeviceHeader_t;


typedef struct RequestHeader_s 
	{
	unsigned char Length;
	unsigned char Unit;
	unsigned char CommandCode;
	unsigned short Status;
	unsigned char Reserved[8];
	} RequestHeader_t;


typedef struct InitPacket_s 
	{
	RequestHeader_t Header;
	unsigned char Data1;
	void _far *Ptr1;
	char _far *CommandLine;
	unsigned char Data2;
	unsigned short MessageFlag;	/* DOS5+, set to have DOS display error msg */
	} InitPacket_t;


typedef struct ReadWritePacket_s 
	{
	RequestHeader_t Header;
	unsigned char MediaDesc;
	char _far *BufferAddress;
	unsigned short ByteCount;
	unsigned short StartingSector;	/* if 0xFFFF, use 'HugeStartingSector' */
	unsigned long VolumeId;
	unsigned long HugeStartingSector;
	} ReadWritePacket_t;


typedef struct IoctlReadWritePacket_s 
	{
	RequestHeader_t Header;
	unsigned char MediaDesc;
	char _far *BufferAddress;
	unsigned short ByteCount;
	} IoctlReadWritePacket_t;


typedef struct OpenClosePacket_s 
	{
	RequestHeader_t Header;
	} OpenClosePacket_t;

typedef struct GenericIoctlPacket_s 
	{
	RequestHeader_t Header;
	unsigned char Category;
	unsigned char Code;
	void _far *Reserved;
	void _far *ParmBufp;		/* I call it a parameter buffer */
	} GenericIoctlPacket_t;

#define SYSFLAG_DONE			0x0100
#define SYSFLAG_BUSY			0x0200
#define SYSFLAG_SYSTEM_ERROR	0x8000

/* Error codes for system errors (low byte of 'rhStatus') */
#define SYSERR_WRITE_PROTECTED			0
#define SYSERR_UNKNOWN_UNIT				1
#define SYSERR_NOT_READY				2
#define SYSERR_UNKNOWN_COMMAND			3
#define SYSERR_CRC						4
#define SYSERR_BAD_LENGTH				5
#define SYSERR_SEEK						6
#define SYSERR_UNKNOWN_MEDIA			7
#define SYSERR_SECTOR_NOT_FOUND			8
#define SYSERR_NO_PAPER					9
#define SYSERR_WRITE_FAULT				10
#define SYSERR_READ_FAULT				11
#define SYSERR_GENERAL_FAILURE			12
#define SYSERR_13						13
#define SYSERR_14						14
#define SYSERR_DISK_CHANGE				15

// Command codes for request packes
#define SYSCMD_INIT						0x00
#define SYSCMD_MEDIA_CHECK				0x01
#define SYSCMD_BUILD_BPB				0x02
#define SYSCMD_IOCTL_READ				0x03
#define SYSCMD_READ						0x04
#define SYSCMD_ND_READ_NO_WAIT			0x05
#define SYSCMD_INPUT_STATUS				0x06
#define SYSCMD_INPUT_FLUSH				0x07
#define SYSCMD_WRITE					0x08
#define SYSCMD_WRITE_VERIFY				0x09
#define SYSCMD_OUTPUT_STATUS			0x0A
#define SYSCMD_OUTPUT_FLUSH				0x0B
#define SYSCMD_IOCTL_WRITE				0x0C
#define SYSCMD_OPEN						0x0D
#define SYSCMD_CLOSE					0x0E
#define SYSCMD_REMOVABLE_MEDIA			0x0F
#define SYSCMD_GENERIC_IOCTL			0x13
#define SYSCMD_GET_LOGICAL_DEVICE		0x17
#define SYSCMD_SET_LOGICAL_DRIVE_MAP	0x18
#define SYSCMD_IOCTL_QUERY				0x19
// Last valid command code
#define SYSCMD_MAX						0x19

#endif

