;---------------------------------------------------------------------------
; HEADER.ASM
;
; This module contains the device driver header and entry points.
; It MUST be the first module to appear in the driver (by specifying
; it as the first file in the linker file list).
;
; It also defines the segment/group ordering.
;
;---------------------------------------------------------------------------

	.seq

; Segment ordering is important in that the device
; header must be at the front of the first segment

_TEXT		SEGMENT  PARA PUBLIC 'CODE'
_TEXT		ENDS
CONST		SEGMENT  PARA PUBLIC 'CONST'
CONST		ENDS
_DATA		SEGMENT  PARA PUBLIC 'DATA'
_DATA		ENDS
c_common	SEGMENT  PARA PUBLIC 'BSS'
c_common	ENDS
_BSS		SEGMENT  PARA PUBLIC 'BSS'
_BSS		ENDS
_INIT_TEXT	SEGMENT PARA PUBLIC 'INICODE'
_INIT_TEXT	ENDS
_INIT_DATA	SEGMENT PARA PUBLIC 'INIDATA'
_INIT_DATA	ENDS


DGROUP	GROUP	_TEXT, CONST, _DATA, c_common, _BSS, _INIT_TEXT, _INIT_DATA



_INIT_DATA	SEGMENT PARA PUBLIC 'INIDATA'
public	_end_data
_end_data	dw	?
_INIT_DATA	ENDS


_INIT_TEXT	SEGMENT PARA PUBLIC 'INICODE'
public	_end_code
_end_code	dw	?
_INIT_TEXT	ENDS


public	_small_code_
_small_code_	= 1234h


_TEXT		SEGMENT  PARA PUBLIC 'CODE'

; Device Driver Header must be the first item in the memory image
	org	0

	public	DeviceHeader

DeviceHeader	dd	-1				; next header
		dw	0C800h				; char, open/close, ioctl
		dw	offset _strat_entry		; point to Strategy Routine
		dw	offset _msdos_entry		; pointer to interrupt routine
		db	'TSPIMGRx'			; Name Field (Must be 8 bytes)
							; (Changed to 'TSPIMGR$' during install)

_TEXT	ends

; Now we can start using the regular assembler segment directives

	.286p
	.MODEL small, c



include dosdrvr.inc


; These routines are in a C module somewhere
extrn	syscmd_init:near
extrn	syscmd_open:near
extrn	syscmd_close:near
extrn	syscmd_ioctl_read:near

	.data

; Command handler jump table
CommandTable	dw	offset syscmd_init			; 00h
		dw	offset syscmd_media_check		; 01h
		dw	offset syscmd_build_bpb			; 02h
		dw	offset syscmd_ioctl_read		; 03h
		dw	offset syscmd_read			; 04h
		dw	offset syscmd_nd_read_no_wait		; 05h
		dw	offset syscmd_input_status		; 06h
		dw	offset syscmd_input_flush		; 07h
		dw	offset syscmd_write			; 08h
		dw	offset syscmd_write_verify		; 09h
		dw	offset syscmd_output_status		; 0Ah
		dw	offset syscmd_output_flush		; 0Bh
		dw	offset syscmd_ioctl_write		; 0Ch
		dw	offset syscmd_open			; 0Dh
		dw	offset syscmd_close			; 0Eh
		dw	offset syscmd_removable_media		; 0Fh
		dw	offset syscmd_invalid			; 10h
		dw	offset syscmd_invalid			; 11h
		dw	offset syscmd_invalid			; 12h
		dw	offset syscmd_generic_ioctl		; 13h
		dw	offset syscmd_invalid			; 14h
		dw	offset syscmd_invalid			; 15h
		dw	offset syscmd_invalid			; 16h
		dw	offset syscmd_get_logical_device	; 17h
		dw	offset syscmd_set_logical_device	; 18h
		dw	offset syscmd_ioctl_query		; 19h
$NumSysCommands = ($ - CommandTable) / 2


; DOS doesn't guarantee how much stack space you'll have, except
; that it will be enough to save the registers.  We need more so
; we'll switch to a local stack while processing driver commands.

dos_stack	dw	256 dup (0)	; local stack space
dos_sp		dw	0		; previous SP
dos_ss		dw	0		; previous SS

	.code

; Variable to store the address of the request header

rhp	label DWORD
rhp_offset	dw	0
rhp_segment	dw	0


;-----------------------------------------------------------------
; DOS calls our _strat_entry routine with the address of a request
; header in ES:BX.  We save this address and wait until DOS calls
; our _msdos_entry routine to process the request.
;-----------------------------------------------------------------

_strat_entry:
	mov	cs:[rhp_offset],bx
	mov	cs:[rhp_segment],es
	retf

_msdos_entry:
	pushf			; save all registers and flags
	push	ax		; (MS-DOS expects them to be saved)
	push	bx
	push	cx
	push	dx
	push	si
	push	di
	push	es
	push	ds

	mov	ax,DGROUP	; set DS = DGROUP for local data addressing
	mov	ds,ax

	mov	[dos_sp],sp	; save current SS:SP
	mov	[dos_ss],ss
	cli
	mov	ss,ax					; switch to local stack 
	mov	sp,offset dos_stack + SIZE dos_stack	; with more room
	sti

	les	bx,cs:[rhp]			; load ptr to request header 
	mov	al,es:[bx].rhCommandCode	; get command code
	cmp	al,$NumSysCommands		; is it within valid range?
	jae	unknown_command			; if not, bail

	sub	ah,ah				; clear high order byte
	shl	ax,1				; double for table index
	mov	si,ax				; place in index register

	push	es				; push address of request
	push	bx				; packet
	call	CommandTable[si]		; call command handler
	add	sp,4				; clean up stack
	jmp	short strat_exit

unknown_command:
	mov	es:[bx].rhStatus,$sysflag_system_error or $sysflag_done or $syserr_unknown_command

strat_exit:
	mov	ax,DGROUP	; set DS = DGROUP for local data addressing
	mov	ds,ax
	cli
	mov	ss,[dos_ss]	; restore MS-DOS stack
	mov	sp,[dos_sp]
	sti

	pop	ds		; restore MS-DOS register state
	pop	es
	pop	di
	pop	si
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	popf
	retf


;---------------------------------------------------------
; Driver commands that we don't implement in this driver
; just set the DONE and ERROR bits, and indicate an 
; unknown command" error
;---------------------------------------------------------
syscmd_ioctl_write:
syscmd_generic_ioctl:
syscmd_read:
syscmd_write:
syscmd_invalid:
syscmd_media_check:
syscmd_build_bpb:
syscmd_nd_read_no_wait:
syscmd_input_status:
syscmd_input_flush:
syscmd_write_verify:
syscmd_output_status:
syscmd_output_flush:
syscmd_removable_media:
syscmd_get_logical_device:
syscmd_set_logical_device:
syscmd_ioctl_query:
	mov	es:[bx].rhStatus,$sysflag_system_error or $sysflag_done or $syserr_unknown_command
	ret

GetEndAddress	proc

	mov	ax,offset _end_code;
	mov	dx,cs
	ret

GetEndAddress	endp



PutChar	proc	value:WORD

	mov	dl,byte ptr [value]
	mov	ah,02h
	int	21h
	ret

PutChar	endp


PutString	proc	p:FAR PTR BYTE

	les	bx,[p]
ops_lp:
	mov	dl,es:[bx]
	cmp	dl,0
	je	ops_endlp
	mov	ah,02h
	int	21h
	inc	bx
	jmp	ops_lp
ops_endlp:
ops_exit:
	ret

PutString	endp

public os_Break
os_Break	proc	val:WORD

	mov	ax,[val]
	int	3
	nop
	ret

os_Break	endp




	end	

