	.model small,c
	.data

OldIrqHandler	dd	0
UserIrqHandler	dw	0
SavedSP		dw	0
SavedSS		dw	0
IrqLevel	db	0
IrqVector	db	0
NestedIrqFlag	dw	0
IrqStackInUse	dw	0
IrqStack	dw	256 dup (?)

	.code


OldTimerHandler	dd	0
TimerTicks	dd	0


TimerTickHandler	proc	far

	pushf
	push	ax
	mov	ax,word ptr cs:[TimerTicks]
	inc	ax
	mov	word ptr cs:[TimerTicks],ax
	or	ax,ax
	jnz	tth_001
	inc	word ptr cs:[TimerTicks+2]
tth_001:
	pop	ax
	popf
	jmp	cs:[OldTimerHandler]

TimerTickHandler	endp
	

GetTickCount	proc

	les	ax,cs:[TimerTicks]	; Load all at once to prevent IRQ
	mov	dx,es			; from changing it while we load
	ret

GetTickCount	endp



DisableInterrupts	proc

	pushf
	pop	ax
	cli
	ret

DisableInterrupts	endp

EnableInterrupts	proc

	pushf
	pop	ax
	sti
	ret

EnableInterrupts	endp

RestoreInterrupts	proc	original_flags:WORD

	test	[original_flags],0200h	; interrupt flag set?
	jz	leave_disabled		; nope, disable interrupts
	sti				; yep, enable interrupts
	jmp	short exit		
leave_disabled:
	cli
exit:
	ret

RestoreInterrupts	endp


ReadIRR	proc

	pushf	
	cli

	mov	al,0Ah
	out	20h,al
	in	al,20h
	sub	ah,ah

	pop	dx
	test	dx,0200h		; interrupt flag set?
	jz	leave_disabled		; nope, disable interrupts
	sti				; yep, enable interrupts
	jmp	short exit		
leave_disabled:
	cli
exit:
	ret

ReadIRR	endp

ReadISR	proc

	pushf	
	cli

	mov	al,0Bh
	out	20h,al
	in	al,20h
	sub	ah,ah

	pop	dx
	test	dx,0200h		; interrupt flag set?
	jz	leave_disabled		; nope, disable interrupts
	sti				; yep, enable interrupts
	jmp	short exit		
leave_disabled:
	cli
exit:
	ret

ReadISR	endp


SendEOI	proc	irq:WORD

	mov	ax,[irq]
	test	al,08h
	jz	first_pic
	and	al,7
	or	al,60h
	out	0A0h,al
	mov	al,2
first_pic:
	and	al,7
	or	al,60h
	out	20h,al
	ret

SendEOI	endp

EnableIrq	proc	irq:WORD

	mov	ax,[irq]
	cmp	al,8
	jb	first_pic
second_pic:
	in	al,21h		; enable redirected IRQ
	and	al,0FDh
	out	21h,al
	mov	cx,[irq]
	and	cx,7
	mov	ah,1
	shl	ah,cl
	not	ah
	in	al,0A1h
	and	al,ah
	out	0A1h,al
	jmp	short ei_exit
first_pic:
	mov	cx,[irq]
	mov	ah,1
	shl	ah,cl
	not	ah
	in	al,021h
	and	al,ah
	out	021h,al
ei_exit:
	ret

EnableIrq	endp

DisableIrq	proc	irq:WORD

	mov	ax,[irq]
	cmp	al,8
	jb	first_pic
second_pic:
	mov	cx,[irq]
	and	cx,7
	mov	ah,1
	shl	ah,cl
	in	al,0A1h
	or	al,ah
	out	0A1h,al
	jmp	short di_exit
first_pic:
	mov	cx,[irq]
	mov	ah,1
	shl	ah,cl
	in	al,021h
	or	al,ah
	out	021h,al
di_exit:
	ret

DisableIrq	endp


NumIrqs	dw	0

GetNumIrqs	proc

	mov	ax,cs:[NumIrqs]
	ret

GetNumIrqs	endp

IrqHandler	proc	far

	inc	cs:[NumIrqs]

	; Save current register values
	push	ax
	push	bx
	push	cx
	push	dx
	push	si
	push	di
	push	bp
	push	es
	push	ds

	; Set up addressing to data segment
	mov	ax,DGROUP
	mov	ds,ax

	; Are we already using the local stack?
	cmp	[IrqStackInUse],0
	je	switch_stacks

	; If so, just note the interrupt and return
	inc	[NestedIrqFlag]
	mov	al,[IrqLevel]
	sub	ah,ah
	push	ax
	call	SendEOI
	add	sp,2
	jmp	exit

switch_stacks:
	; Switch to our local stack
	mov	[NestedIrqFlag],0
	mov	[IrqStackInUse],1
	mov	[SavedSP],sp
	mov	[SavedSS],ss
	mov	ax,ds
	mov	ss,ax
	mov	sp,offset IrqStack + size IrqStack

	; Call the C language handler
	mov	al,[IrqLevel]
	sub	ah,ah
	push	ax
	call	[UserIrqHandler]
	add	sp,2

	; Restore the original stack
	cli
	mov	ss,[SavedSS]
	mov	sp,[SavedSP]
	mov	[IrqStackInUse],0

exit:
	; Restore saved registers and exit
	pop	ds
	pop	es
	pop	bp
	pop	di
	pop	si
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	iret

IrqHandler	endp


InstallIrqHandler	proc	irq:WORD, handler:NEAR PTR

	mov	ax,word ptr [handler]
	mov	word ptr [UserIrqHandler],ax

	mov	ax,[irq]
	mov	[IrqLevel],al
	cmp	al,8
	jb	low_irq
	and	al,07h
	add	al,70h
	jmp	short irq_ok
low_irq:
	add	al,8
irq_ok:
	mov	[IrqVector],al

	; save current interrupt vector
	mov	al,[IrqVector]
	mov	ah,35h
	int	21h
	mov	word ptr [OldIrqHandler],bx
	mov	word ptr [OldIrqHandler+2],es

	; install our interrupt vector
	mov	al,[IrqVector]
	push	ds
	mov	dx,cs
	mov	ds,dx
	mov	dx,offset IrqHandler
	mov	ah,25h
	int	21h
	pop	ds

	; return success
	mov	ax,1
	ret

InstallIrqHandler	endp


RemoveIrqHandler	proc	irq:WORD

	push	ds
	mov	al,[IrqVector]
	lds	dx,[OldIrqHandler]
	mov	ah,25h
	int	21h
	pop	ds
	ret

RemoveIrqHandler	endp



InstallTimer	proc

	; save current interrupt vector
	mov	al,8
	mov	ah,35h
	int	21h
	mov	word ptr cs:[OldTimerHandler],bx
	mov	word ptr cs:[OldTimerHandler+2],es

	; install our interrupt vector
	push	ds
	mov	ax,cs
	mov	ds,ax
	mov	dx,offset TimerTickHandler
	mov	al,8
	mov	ah,25h
	int	21h
	pop	ds

	; return success
	mov	ax,1
	ret

InstallTimer	endp



RemoveTimer	proc

	push	ds
	mov	al,8
	lds	dx,cs:[OldTimerHandler]
	mov	ah,25h
	int	21h
	pop	ds
	ret

RemoveTimer	endp

	end

