
/***********************************************************************

NCR_400.H		NCR 53C400 I/O routines

We also define the various SCSI_PHASE_xxx and SCSI_BIT_xxx constants
here.  This allows the adapter module to use any convenient representation
for these.  For the NCR 53C400 the bits are ordered just as they appear
in the ISR and CSC registers.

***********************************************************************/


#define SCSI_PHASE_BUS_FREE	0
#define SCSI_PHASE_CMD		1
#define SCSI_PHASE_STATUS	2
#define SCSI_PHASE_MSG_OUT	3
#define SCSI_PHASE_MSG_IN	4
#define SCSI_PHASE_DATA_OUT	5
#define SCSI_PHASE_DATA_IN	6

#define SCSI_BIT_ATN	0x0200
#define SCSI_BIT_ACK	0x0100
#define SCSI_BIT_RST	0x0080
#define SCSI_BIT_BSY	0x0040
#define SCSI_BIT_REQ	0x0020
#define SCSI_BIT_MSG	0x0010
#define SCSI_BIT_CD		0x0008
#define SCSI_BIT_IO		0x0004
#define SCSI_BIT_SEL	0x0002
#define SCSI_BIT_DBP	0x0001


int		adapter_Init(char FAR *cmdline);	// Starts up the adapter
void	adapter_Term();						// Shuts down the adapter

void	adapter_Poll();						// Called while waiting for an event

int		adapter_GetScsiId();				// Returns adapter's SCSI ID

int		adapter_GetScsiBits();				// Returns current SCSI BUS state
											// (OR'ed SCSI_BIT_xxx bits)


void	adapter_SelectPhase(int phase);		// Selects a particular bus phase

int		adapter_Read( int phase, 			// Reads data from the initiator
					  void FAR *bufp, 		// during the specified bus phase
					  int maxbytes);		// (phase is selected before read)

int		adapter_Write( int phase, 			// Writes data to the initiator
					   void FAR *bufp,	 	// during the specified bus phase
					   int maxbytes);		// (phase is selected before write)

void	adapter_Disconnect(void);			// Disconnects adapter from the
											// SCSI bus

int		adapter_Reconnect(int initiator);	// Reconnects to the specified
											// initiator.

void	adapter_Reset();					// Resets and re-enables the
											// adapter
