#if defined(__ZTC__)
#pragma ZTC align 1
#elif defined(__BORLANDC__)
#pragma option -a-
#else
#pragma pack(1)
#endif

/* SCSI messages that we can handle */
#define SCSI_Msg_CommandComplete		0x00
#define SCSI_Msg_Extended				0x01
#define SCSI_Msg_Abort					0x06
#define SCSI_Msg_BusDeviceReset			0x0C
#define SCSI_Msg_InitiatorDetectedError	0x05
#define SCSI_Msg_Nop					0x08
#define SCSI_Msg_Reject					0x07

/* The IDENTIFY message contains the following bits */
#define SCSI_Msg_IdentifyBit			0x80
#define SCSI_Msg_IdentifyDiscPrivBit	0x40
#define SCSI_Msg_IdentifyLunTarBit		0x20
#define SCSI_Msg_IdentifyLunTarMask		0x07

/* SCSI Status codes */
#define SCSI_Status_Good						0x00
#define SCSI_Status_CheckCondition				0x02
#define SCSI_Status_ConditionMet				0x04
#define SCSI_Status_Busy						0x08
#define SCSI_Status_Intermediate				0x10
#define SCSI_Status_ReservationConflict			0x18
#define SCSI_Status_CommandTerminated			0x21
#define SCSI_Status_QueueFull					0x28


/* CDB command codes */
#define SCSI_Cmd_TestUnitReady		0x00
#define SCSI_Cmd_Rewind				0x01
#define SCSI_Cmd_RequestSense		0x03
#define SCSI_Cmd_ReadBlockLimits	0x05
#define SCSI_Cmd_Read				0x08
#define SCSI_Cmd_Write				0x0A
#define SCSI_Cmd_WriteFilemarks		0x10
#define SCSI_Cmd_Space 				0x11
#define SCSI_Cmd_Inquiry			0x12
#define SCSI_Cmd_ModeSelect			0x15
#define SCSI_Cmd_ReserveUnit		0x16
#define SCSI_Cmd_ReleaseUnit		0x17
#define SCSI_Cmd_Erase				0x19
#define SCSI_Cmd_ModeSense			0x1A
#define SCSI_Cmd_LoadUnload			0x1B
#define SCSI_Cmd_SendDiagnostic		0x1D
#define SCSI_Cmd_PreventAllowRemoval		0x1E

/* Sense Key values */
#define SCSI_Skey_NoSense			0x00
#define SCSI_Skey_RecoveredError	0x01
#define SCSI_Skey_NotReady			0x02
#define SCSI_Skey_MediaError		0x03
#define SCSI_Skey_HardwareError		0x04
#define SCSI_Skey_IllegalRequest	0x05
#define SCSI_Skey_UnitAttention		0x06
#define SCSI_Skey_DataProtect		0x07
#define SCSI_Skey_BlankCheck		0x08
#define SCSI_Skey_VendorSpecific	0x09
#define SCSI_Skey_CopyAborted		0x0A
#define SCSI_Skey_AbortedCommand	0x0B
#define SCSI_Skey_Equal				0x0C
#define SCSI_Skey_VolumeOverflow	0x0D
#define SCSI_Skey_Miscompare		0x0E
#define SCSI_Skey_Reserved			0x0F

/* ASC codes */
#define SCSI_Asc_LunNotReady				0x04
#define SCSI_Asc_InvalidCommand				0x20
#define SCSI_Asc_InvalidFieldInCdb			0x24
#define SCSI_Asc_LunNotSupported			0x25
#define SCSI_Asc_MediaFormatCorrupted		0x31
#define SCSI_Asc_WriteProtected				0x27
#define SCSI_Asc_WriteError					0x0C
#define SCSI_Asc_ReadError					0x11
#define SCSI_Asc_DataPhaseError				0x4B
#define SCSI_Asc_ScsiInterfaceFailure		0x54




/* ********************* WARNING ********************************** */
/* The structures below expect that the compiler lays out bitfields */
/* LSB first.														*/
/* **************************************************************** */

typedef struct SCSI_Cdb_Generic6_s {
	unsigned int CommandCode	: 8;
	unsigned int Reserved1		: 5;
	unsigned int Lun			: 3;
	unsigned int Reserved2		: 8;
	unsigned int Reserved3		: 8;
	unsigned int Reserved4		: 8;
	unsigned int Link			: 1;
	unsigned int Flag			: 1;
	unsigned int Reserved5		: 4;
	unsigned int VendorSpecific	: 2;
	} SCSI_Cdb_Generic6_t;

typedef struct SCSI_Cdb_TestUnitReady_s {
	unsigned int CommandCode	: 8;	/* should be 0x00 */
	unsigned int 				: 5;	/* must be 0 */
	unsigned int Lun			: 3;
	unsigned int 				: 8;	/* must be 0 */
	unsigned int 				: 8;	
	unsigned int 				: 8;
	unsigned int Link			: 1;
	unsigned int Flag			: 1;
	unsigned int 				: 4;
	unsigned int VendorSpecific	: 2;
	} SCSI_Cdb_TestUnitReady_t;

typedef struct SCSI_Cdb_Rewind_s {
	unsigned int CommandCode	: 8;	/* should be 0x01 */
	unsigned int Immediate		: 1;
	unsigned int 				: 4;
	unsigned int Lun			: 3;
	unsigned int 				: 8;
	unsigned int 				: 8;	
	unsigned int 				: 8;
	unsigned int Link			: 1;
	unsigned int Flag			: 1;
	unsigned int 				: 4;
	unsigned int VendorSpecific	: 2;
	} SCSI_Cdb_Rewind_t;

typedef struct SCSI_Cdb_RequestSense_s {
	unsigned int CommandCode		: 8;	/* should be 0x03 */
	unsigned int 					: 5;	/* must be 0 */
	unsigned int Lun				: 3;
	unsigned int 					: 8;	/* must be 0 */
	unsigned int 					: 8;	
	unsigned int AllocationLength	: 8;
	unsigned int Link				: 1;
	unsigned int Flag				: 1;
	unsigned int 					: 4;
	unsigned int VendorSpecific		: 2;
	} SCSI_Cdb_RequestSense_t;

typedef struct SCSI_SenseData_s {
	unsigned int ErrorCode				: 7;
	unsigned int Valid					: 1;
	unsigned int SegmentNumber			: 8;
	unsigned int SenseKey				: 4;
	unsigned int 						: 1;
	unsigned int IllegalLength			: 1;
	unsigned int EndOfMedia				: 1;
	unsigned int FileMark				: 1;
	unsigned int InformationByte3		: 8;
	unsigned int InformationByte2		: 8;
	unsigned int InformationByte1		: 8;
	unsigned int InformationByte0		: 8;
	unsigned int AdditionalLength		: 8;
	unsigned int 						: 8;
	unsigned int 						: 8;
	unsigned int 						: 8;
	unsigned int 						: 8;
	unsigned int Asc					: 8;
	unsigned int Asq					: 8;
	unsigned int 						: 8;
	unsigned int BitPointer				: 3;
	unsigned int BitPointerValid		: 1;
	unsigned int						: 2;
	unsigned int CdbError				: 1;
	unsigned int SenseKeySpecificValid	: 1;
	unsigned int SenseKeySpecific1		: 8;
	unsigned int SenseKeySpecific0		: 8;
	} SCSI_SenseData_t;


typedef struct SCSI_Cdb_ReadBlockLimits_s {
	unsigned int CommandCode	: 8;	/* should be 0x05 */
	unsigned int 				: 5;
	unsigned int Lun			: 3;
	unsigned int 				: 8;
	unsigned int 				: 8;	
	unsigned int 				: 8;
	unsigned int Link			: 1;
	unsigned int Flag			: 1;
	unsigned int 				: 4;
	unsigned int VendorSpecific	: 2;
	} SCSI_Cdb_ReadBlockLimits_t;

typedef struct SCSI_BlockLimits_s {
	unsigned int					: 8;
	unsigned int MaxBlockSize2		: 8;
	unsigned int MaxBlockSize1		: 8;
	unsigned int MaxBlockSize0		: 8;
	unsigned int MinBlockSize1		: 8;
	unsigned int MinBlockSize0		: 8;
	} SCSI_BlockLimits_t;

typedef struct SCSI_Cdb_Read_s {
	unsigned int CommandCode			: 8;	/* should be 0x08 */
	unsigned int Fixed					: 1;
	unsigned int SuppressIllegalLength	: 1;
	unsigned int 						: 3;
	unsigned int Lun					: 3;
	unsigned int Count2					: 8;
	unsigned int Count1					: 8;	
	unsigned int Count0					: 8;
	unsigned int Link					: 1;
	unsigned int Flag					: 1;
	unsigned int 						: 4;
	unsigned int VendorSpecific			: 2;
	} SCSI_Cdb_Read_t;
	

typedef struct SCSI_Cdb_Write_s {
	unsigned int CommandCode			: 8;	/* should be 0x0A */
	unsigned int Fixed					: 1;
	unsigned int 						: 4;
	unsigned int Lun					: 3;
	unsigned int Count2					: 8;
	unsigned int Count1					: 8;	
	unsigned int Count0					: 8;
	unsigned int Link					: 1;
	unsigned int Flag					: 1;
	unsigned int 						: 4;
	unsigned int VendorSpecific			: 2;
	} SCSI_Cdb_Write_t;
	

typedef struct SCSI_Cdb_WriteFilemarks_s {
	unsigned int CommandCode	: 8;	/* should be 0x10 */
	unsigned int Immediate		: 1;
	unsigned int Setmark		: 1;
	unsigned int 				: 3;
	unsigned int Lun			: 3;
	unsigned int Count2			: 8;
	unsigned int Count1			: 8;	
	unsigned int Count0			: 8;
	unsigned int Link			: 1;
	unsigned int Flag			: 1;
	unsigned int 				: 4;
	unsigned int VendorSpecific	: 2;
	} SCSI_Cdb_WriteFilemarks_t;
	
typedef struct SCSI_Cdb_Space_s {
	unsigned int CommandCode	: 8;	/* should be 0x11 */
	unsigned int Code			: 3;
	unsigned int 				: 2;
	unsigned int Lun			: 3;
	unsigned int Count2			: 8;
	unsigned int Count1			: 8;	
	unsigned int Count0			: 8;
	unsigned int Link			: 1;
	unsigned int Flag			: 1;
	unsigned int 				: 4;
	unsigned int VendorSpecific	: 2;
	} SCSI_Cdb_Space_t;
	
typedef struct SCSI_Cdb_Inquiry_s {
	unsigned int CommandCode		: 8;	/* should be 0x12 */
	unsigned int Evpd				: 1;
	unsigned int 					: 4;
	unsigned int Lun				: 3;
	unsigned int PageCode			: 8;
	unsigned int TandbergPageCode   : 8;	
	unsigned int AllocationLength	: 8;
	unsigned int Link				: 1;
	unsigned int Flag				: 1;
	unsigned int 					: 4;
	unsigned int VendorSpecific		: 2;
	} SCSI_Cdb_Inquiry_t;

typedef struct SCSI_InquiryData_s {
	unsigned int DeviceType					: 5;
	unsigned int DeviceQualifier			: 3;
	unsigned int DeviceModifier				: 7;
	unsigned int RemovableMedia				: 1;
	unsigned int AnsiVersion				: 3;
	unsigned int EcmaVersion				: 3;
	unsigned int IsoVersion					: 2;
	unsigned int ResponseDataFormat			: 4;
	unsigned int 							: 2;
	unsigned int TermIoProcessSupport		: 1;
	unsigned int AsyncEventNotifySupport	: 1;
	unsigned int AdditionalLength			: 8;
	unsigned int							: 8;
	unsigned int							: 8;
	unsigned int SoftResetSupport			: 1;
	unsigned int CommandQueueSupport		: 1;
	unsigned int 							: 1;
	unsigned int LinkedCommandSupport		: 1;
	unsigned int SynchronousTransferSupport : 1;
	unsigned int WideBus16Support			: 1;
	unsigned int WideBus32Support			: 1;
	unsigned int RelativeAddressingSupport	: 1;
	unsigned char VendorId[8];
	unsigned char ProductId[16];
	unsigned char ProductRevisionLevel[4];	
	unsigned char VendorSpecific[16];
	} SCSI_InquiryData_t;

typedef struct SCSI_Cdb_Erase_s {
	unsigned int CommandCode	: 8;	/* should be 0x01 */
	unsigned int Long			: 1;
	unsigned int Immediate		: 1;
	unsigned int 				: 3;
	unsigned int Lun			: 3;
	unsigned int 				: 8;
	unsigned int 				: 8;	
	unsigned int 				: 8;
	unsigned int Link			: 1;
	unsigned int Flag			: 1;
	unsigned int 				: 4;
	unsigned int VendorSpecific	: 2;
	} SCSI_Cdb_Erase_t;

typedef struct SCSI_Cdb_LoadUnload_s {
	unsigned int CommandCode	: 8;	/* should be 0x01 */
	unsigned int Immediate		: 1;
	unsigned int r1				: 4;
	unsigned int Lun			: 3;
	unsigned int r2				: 8;
	unsigned int r3				: 8;	
	unsigned int Load			: 1;
	unsigned int Retension		: 1;
	unsigned int EndOfTape		: 1;
	unsigned int 				: 5;
	unsigned int Link			: 1;
	unsigned int Flag			: 1;
	unsigned int 				: 4;
	unsigned int VendorSpecific	: 2;
	} SCSI_Cdb_LoadUnload_t;


typedef struct SCSI_Cdb_ModeSense_s {
	unsigned int CommandCode				: 8;	/* should be 0x1A */
	unsigned int 							: 3;
	unsigned int DisableBlockDescriptors	: 1;
	unsigned int 							: 1;
	unsigned int Lun						: 3;
	unsigned int PageCode					: 6;
	unsigned int PageControl				: 2;
	unsigned int 							: 8;	
	unsigned int AllocationLength			: 8;
	unsigned int Link						: 1;
	unsigned int Flag						: 1;
	unsigned int 							: 4;
	unsigned int VendorSpecific				: 2;
	} SCSI_Cdb_ModeSense_t;

typedef struct SCSI_ModeSenseData_s {
	unsigned int SenseDataLength			: 8;
	unsigned int MediumType					: 8;
	unsigned int Speed						: 4;
	unsigned int BufferedMode				: 3;
	unsigned int WriteProtected				: 1;
	unsigned int BlockDescriptorLength		: 8;
	unsigned int DensityCode				: 8;
	unsigned int NumBlocks2					: 8;
	unsigned int NumBlocks1					: 8;
	unsigned int NumBlocks0					: 8;
	unsigned int 							: 8;
	unsigned int BlockLength2				: 8;
	unsigned int BlockLength1				: 8;
	unsigned int BlockLength0				: 8;
	} SCSI_ModeSenseData_t;

typedef struct SCSI_Cdb_ModeSelect_s {
	unsigned int CommandCode				: 8;	/* should be 0x1A */
	unsigned int SavePages					: 1;
	unsigned int r1							: 3;
	unsigned int PageFormat					: 1;
	unsigned int Lun						: 3;
	unsigned int r2							: 8;	
	unsigned int r3							: 8;	
	unsigned int ParameterListLength		: 8;
	unsigned int Link						: 1;
	unsigned int Flag						: 1;
	unsigned int r4							: 4;
	unsigned int VendorSpecific				: 2;
	} SCSI_Cdb_ModeSelect_t;

typedef struct SCSI_ModeSelectData_s {
	unsigned int 							: 8;
	unsigned int 							: 8;
	unsigned int Speed						: 4;
	unsigned int BufferedMode				: 3;
	unsigned int 							: 1;
	unsigned int BlockDescriptorLength		: 8;
	unsigned int DensityCode				: 8;
	unsigned int NumBlocks2					: 8;
	unsigned int NumBlocks1					: 8;
	unsigned int NumBlocks0					: 8;
	unsigned int 							: 8;
	unsigned int BlockLength2				: 8;
	unsigned int BlockLength1				: 8;
	unsigned int BlockLength0				: 8;
	} SCSI_ModeSelectData_t;

	

typedef struct SCSI_Cdb_ReserveUnit_s {
	unsigned int CommandCode				: 8;	/* should be 0x16 */
	unsigned int Extent						: 1;
	unsigned int ThirdPartyDeviceId			: 3;
	unsigned int ThirdPartyFlag				: 1;
	unsigned int Lun						: 3;
	unsigned int ReservationId				: 8;	
	unsigned int ExtentListLength1			: 8;	
	unsigned int ExtentListLength0			: 8;	
	unsigned int Link						: 1;
	unsigned int Flag						: 1;
	unsigned int 							: 4;
	unsigned int VendorSpecific				: 2;
	} SCSI_Cdb_ReserveUnit_t;

typedef struct SCSI_Cdb_ReleaseUnit_s {
	unsigned int CommandCode				: 8;	/* should be 0x17 */
	unsigned int Extent						: 1;
	unsigned int ThirdPartyDeviceId			: 3;
	unsigned int ThirdPartyFlag				: 1;
	unsigned int Lun						: 3;
	unsigned int ReservationId				: 8;	
	unsigned int 							: 8;	
	unsigned int 							: 8;	
	unsigned int Link						: 1;
	unsigned int Flag						: 1;
	unsigned int 							: 4;
	unsigned int VendorSpecific				: 2;
	} SCSI_Cdb_ReleaseUnit_t;

	
typedef struct SCSI_Cdb_SendDiagnostic_s {
	unsigned int CommandCode		: 8;	/* should be 0x12 */
	unsigned int UnitOfflineOk		: 1;
	unsigned int DeviceOfflineOk	: 1;
	unsigned int SelfTest			: 1;
	unsigned int 					: 1;
	unsigned int PageFormat			: 1;
	unsigned int Lun				: 3;
	unsigned int 					: 8;	
	unsigned int ParmListLenByte1	: 8;
	unsigned int ParmListLenByte0	: 8;
	unsigned int Link				: 1;
	unsigned int Flag				: 1;
	unsigned int 					: 4;
	unsigned int VendorSpecific		: 2;
	} SCSI_Cdb_SendDiagnostic_t;


typedef struct SCSI_Cdb_PreventAllowRemoval_s {
	unsigned int CommandCode	: 8;	/* should be 0x05 */
	unsigned int 				: 5;
	unsigned int Lun			: 3;
	unsigned int 				: 8;
	unsigned int 				: 8;	
	unsigned int Prevent		: 1;
	unsigned int 				: 7;
	unsigned int Link			: 1;
	unsigned int Flag			: 1;
	unsigned int 				: 4;
	unsigned int VendorSpecific	: 2;
	} SCSI_Cdb_PreventAllowRemoval_t;

#if defined(__ZTC__)
#pragma ZTC align
#elif defined(__BORLANDC__)
#pragma options -a.
#else
#pragma pack()
#endif

