
/*********************************************************
* TSPIDRVR.C
* 
* This module implements the DOS device driver routines 
* required for the TSPI manager. 
* 
* The open/close routines are empty shells.  
* 
* The IOCTL read routine returns the address of the 
* TSPI manager's entry point.
* 
* The init routine retrieves the command line parameters
* and initializes the 53C400A.
* 
*********************************************************/


#include "dosdrvr.h"
#include "header.h"
#include "scsi.h"
#include "tspi.h"
#include "tspilink.h"


void syscmd_ioctl_read( IoctlReadWritePacket_t FAR *p )
	{

	/* Return the TSPI entry point address in the caller's buffer */
	if (p->ByteCount >= 4)
		{
		*(long FAR *)p->BufferAddress = (long) (void FAR *) tspi_Entry;
		p->ByteCount = 4;
		}
	else
		{
		extern void os_Break(int x);
		os_Break(p->ByteCount);
		p->ByteCount = 0;
		}

	p->Header.Status = SYSFLAG_DONE;
	}


void syscmd_open( OpenClosePacket_t FAR *p )
	{
	p->Header.Status = SYSFLAG_DONE;
	}


void syscmd_close( OpenClosePacket_t FAR *p )
	{
	p->Header.Status = SYSFLAG_DONE;
	}



void syscmd_init( InitPacket_t FAR *p )
	{
	int ok;
	int i;

	p->Data1 = 0;

	PutString("TSPIMGR$ Version 1.04\r\n");
	PutString("Command Line: '");
	for (i=0; *(p->CommandLine+i) && *(p->CommandLine+i) != 13; i++)
		{
		PutChar(*(p->CommandLine+i));
		}
	PutString("'\r\n");

	InstallTimer();

	ok = tspi_Init(p->CommandLine);

	p->CommandLine = (void FAR *) 0;	/* required for DOS char devs */

	if ( !ok )
		{
		RemoveTimer();
		PutString("Driver not installed!\r\n");
		p->Ptr1 = (void FAR *) ((long)GetEndAddress() & 0xFFFF0000);
		p->Header.Status = SYSFLAG_SYSTEM_ERROR | SYSFLAG_DONE | SYSERR_GENERAL_FAILURE;
		}
	else
		{
		DeviceHeader.Name[7] = '$';		// Fix driver name
		p->Ptr1 = GetEndAddress();
		p->Header.Status = SYSFLAG_DONE;
		}
	}




