#ifndef TSPI_H_INCLUDED
#define TSPI_H_INCLUDED	0x0104

typedef struct TSPI_EVENT_s				// Holds incoming SCSI 
	{									// CDBs and bus events
	unsigned char	CommandCode;		// Type of command
	unsigned char	Error;				// Returns error status
	unsigned short	Flags;				// TSPI_FLAG_xxxx
	unsigned char	AdapterIndex;		// Adapter number 
	unsigned char	InitiatorId;		// Who sent it
	unsigned char	Lun;				// Our LUN
	unsigned char	Reserved1[15];		// Reserved for API use
	unsigned long	Timeout;			// In milliseconds
	unsigned char	Reserved2[3];		// Reserved (alignment)
	unsigned char	CdbLength;			// Length of CDB
	unsigned char	CdbByte[16];		// CDB data bytes
	} TSPI_EVENT;

typedef struct TSPI_CMD_s				// Generic TSPI cmd
	{
	unsigned char	CommandCode;		// Type of command
	unsigned char	Error;				// Returns error status
	unsigned short	Flags;				// TSPI_FLAG_xxxx
	unsigned char	AdapterIndex;		// Adapter number 
	unsigned char	InitiatorId;		// Host SCSI ID
	unsigned char	Lun;				// Our LUN
	unsigned char	Reserved1[15];		// Reserved for API use
	unsigned long	Parm[6];			// Generic parameters
	} TSPI_CMD;

typedef struct TSPI_XFER_s				// Data transfer cmd
	{
	unsigned char	CommandCode;		// Type of command
	unsigned char	Error;				// Returns error status
	unsigned short	Flags;				// TSPI_FLAG_xxxx
	unsigned char	AdapterIndex;		// Adapter number 
	unsigned char	InitiatorId;		// Host SCSI ID
	unsigned char	Lun;				// Our LUN
	unsigned char	Reserved1[15];		// Reserved for API use
	unsigned long	TransferLength;		// Bytes to read/write
	void FAR *		TransferAddress;	// Data buffer address
	unsigned long	ResidualLength;		// Bytes NOT sent
	unsigned long	Reserved2[3];		// Reserved (alignment)
	} TSPI_XFER;

// Flags
#define TSPI_FLAG_BusReset			0x0001
#define TSPI_FLAG_DeviceReset		0x0002
#define TSPI_FLAG_HostMsgWaiting	0x0004
#define TSPI_FLAG_SaveDataPointer	0x0008
#define TSPI_FLAG_TargetRoutine		0x0020
#define TSPI_FLAG_DiscPriv			0x0040

// Command Codes
#define TSPI_CMD_AdapterInfo		0
#define TSPI_CMD_AttachLUN			1
#define TSPI_CMD_DetachLUN			2
#define TSPI_CMD_GetEvent			3
#define TSPI_CMD_ReadFromHost		4
#define TSPI_CMD_WriteToHost		5
#define TSPI_CMD_CompleteCommand	6
#define TSPI_CMD_SendMessage		7
#define TSPI_CMD_GetMessage			8
#define TSPI_CMD_Disconnect			9
#define TSPI_CMD_Reconnect			10

// Error Codes
#define TSPI_ERR_None				0
#define TSPI_ERR_InvalidCommand		1
#define TSPI_ERR_Busy				2
#define TSPI_ERR_InvalidAdapter		3
#define TSPI_ERR_InvalidTarget		4
#define TSPI_ERR_InvalidLUN			5
#define TSPI_ERR_LunNotAvailable	6
#define TSPI_ERR_Timeout			7
#define TSPI_ERR_Initiator			8
#define TSPI_ERR_TooLarge			9

typedef int (CDECL FAR *TSPI_ENTRY)(void FAR *p);

#endif

