
#include <stdio.h>
#include <conio.h>

#include "tspi.h"
#include "tspi_dos.h"
#include "scsidefs.h"

int (FAR *tspi_SendCommand)(void FAR *p);	// ptr to TSPI entry routine

int Ready;
int UnitAttention;

#define SENSE_LEN	18
unsigned char SenseData[SENSE_LEN];

#define INQUIRY_LEN	36
unsigned char InquiryData[INQUIRY_LEN] =
	{
	0x03,			// processor device
	0x00,			// device qualifier
	0x02,			// ANSI version 2 
	0x02,			// Response format 2
	0x20,			// 32 additional bytes
	0x00,			// Reserved
	0x00,			// Reserved
	0x00,			// Attributes (wide, sync, etc.)
	'T',			// Vendor ID ("TSPI    ")
	'S',
	'P',
	'I',
	' ',
	' ',
	' ',
	' ',
	'S',			// Product ID ("SAMPLE-DEVICE   ");
	'A',
	'M',
	'P',
	'L',
	'E',
	'-',
	'D',
	'E',
	'V',
	'I',
	'C',
	'E',
	' ',
	' ',
	' ',
	'1',			// Product Revision ("1.04")
	'.',
	'0',
	'4'
	};


void SetSenseData(int skey, int asc, int asq)
	{
	SenseData[0] = 0x70;		// Current error
	SenseData[1] = 0;			// Segment
	SenseData[2] = (SenseData[2] & 0xF0) | (skey & 0x0F);
	SenseData[3] = 0;
	SenseData[4] = 0;
	SenseData[5] = 0;
	SenseData[6] = 0;
	SenseData[7] = SENSE_LEN - 7;
	SenseData[8] = 0;
	SenseData[9] = 0;
	SenseData[10] = 0;
	SenseData[11] = 0;
	SenseData[12] = asc;
	SenseData[13] = asq;
	SenseData[14] = 0;
	SenseData[15] = 0;
	SenseData[16] = 0;
	SenseData[17] = 0;
	}




int TestUnitReady(TSPI_EVENT *event)
	{
	// We're always ready
	if (Ready)
		return SCSI_Status_Good;
	else
		{
		SetSenseData(SCSI_Skey_NotReady,0,0);
		return SCSI_Status_CheckCondition;
		}
	}


int RequestSense(TSPI_EVENT *event)
	{
	TSPI_XFER	xfer;

	xfer.CommandCode = TSPI_CMD_WriteToHost;
	xfer.AdapterIndex = event->AdapterIndex;
	xfer.InitiatorId = event->InitiatorId;
	xfer.Lun = event->Lun;
	if (event->CdbByte[4] > SENSE_LEN)
		xfer.TransferLength = SENSE_LEN;
	else
		xfer.TransferLength = event->CdbByte[4];
	xfer.TransferAddress = &SenseData[0];
	tspi_SendCommand(&xfer);
	if (xfer.Error)
		{
		SetSenseData(SCSI_Skey_HardwareError,SCSI_Asc_ScsiInterfaceFailure,0);
		return SCSI_Status_CheckCondition;
		}

	return SCSI_Status_Good;
	}


int Inquiry(TSPI_EVENT *event)
	{
	TSPI_XFER	xfer;

	xfer.CommandCode = TSPI_CMD_WriteToHost;
	xfer.AdapterIndex = event->AdapterIndex;
	xfer.InitiatorId = event->InitiatorId;
	xfer.Lun = event->Lun;
	if (event->CdbByte[4] > INQUIRY_LEN)
		xfer.TransferLength = INQUIRY_LEN;
	else
		xfer.TransferLength = event->CdbByte[4];
	xfer.TransferAddress = &InquiryData[0];
	tspi_SendCommand(&xfer);
	if (xfer.Error)
		{
		printf("Inquiry:  transfer error\n");
		SetSenseData(SCSI_Skey_HardwareError,SCSI_Asc_ScsiInterfaceFailure,0);
		return SCSI_Status_CheckCondition;
		}

	return SCSI_Status_Good;
	}


void ProcessEvent(TSPI_EVENT *event)
	{
	TSPI_CMD	cmd;
	int			status;
	int			i;

	if (event->Flags & (TSPI_FLAG_BusReset | TSPI_FLAG_DeviceReset))
		{
		// If we get a reset, set a flag to send a unit attention
		// check condition back to the host on the next command
		UnitAttention = 1;
		}

	if (event->CdbLength > 0)
		{
		printf("Received CDB: ");
		for (i=0; i<event->CdbLength; i++)
			printf("%02X ",event->CdbByte[i]);
		printf("\n");

		if (   UnitAttention 
			&& (event->CdbByte[0] != SCSI_Cmd_RequestSense)
			&& (event->CdbByte[0] != SCSI_Cmd_Inquiry) )
			{
			// Terminate the command with a check condition status
			status = SCSI_Status_CheckCondition;
			}
		else
			{
			switch (event->CdbByte[0])
				{
				case SCSI_Cmd_TestUnitReady:
					status = TestUnitReady(event);
					break;

				case SCSI_Cmd_Inquiry:
					status = Inquiry(event);
					break;

				case SCSI_Cmd_RequestSense:
					if (UnitAttention)
						{
						// Report the unit attention sense data
						SetSenseData(SCSI_Skey_UnitAttention,0,0);
						UnitAttention = 0;	// clear unit attention condition
						}
					status = RequestSense(event);
					break;

				default:
					// Unsupported command
					SetSenseData(SCSI_Skey_IllegalRequest,SCSI_Asc_InvalidCommand,0);
					status = SCSI_Status_CheckCondition;
				}
			}

		printf("Completing command with status %02X\n",status);

		// Finish the command by sending status and disconnecting
		cmd.CommandCode = TSPI_CMD_CompleteCommand;
		cmd.AdapterIndex = event->AdapterIndex;
		cmd.Lun = event->Lun;
		cmd.Parm[0] = status;
		tspi_SendCommand(&cmd);
		}
	}





void RunTarget(unsigned char adapter, unsigned char lun)
	{
	TSPI_CMD	cmd;
	TSPI_EVENT	event;

	// First get the adapter information (to display SCSI ID below)
	cmd.CommandCode = TSPI_CMD_AdapterInfo;
	cmd.AdapterIndex = 0;
	tspi_SendCommand(&cmd);
	if (cmd.Error != TSPI_ERR_None)
		{
		printf("Unable to retrieve TSPI adapter %u information!\n",adapter);
		return;
		}

	printf("Attaching to adapter %u LUN %u (SCSI ID %lu)\n",adapter,lun,cmd.Parm[2]);

	// Attach to the specified LUN
	cmd.CommandCode = TSPI_CMD_AttachLUN;
	cmd.AdapterIndex = adapter;
	cmd.Lun = lun;;
	tspi_SendCommand(&cmd);
	if (cmd.Error)
		{
		printf("Unable to attach LUN, error %d\n",cmd.Error);
		return;
		}

	printf("Press any key to exit\n");

	// Flag a "unit attention" condition to let the initiator
	// know that we've just restarted.
	UnitAttention = 1;
	SetSenseData(SCSI_Skey_UnitAttention,0,0);

	// Process incoming SCSI events until the user presses a key
	while (!kbhit())
		{
		// First get an event
		event.CommandCode = TSPI_CMD_GetEvent;	// get SCSI event
		event.AdapterIndex = adapter;			// this adapter
		event.Lun = lun;						// this lun
		event.Timeout = 100;					// milliseconds
		tspi_SendCommand(&event);
		if (event.Error == TSPI_ERR_None)		// got one?
			ProcessEvent(&event);				// process it
		else 
			{
			// Ignore timeout errors, but quit on any other
			if (event.Error != TSPI_ERR_Timeout)
				{
				printf("Unable to get SCSI bus event, error %d\n",event.Error);
				break;
				}
			}
		}

	// Detach LUN from the TSPI manager
	printf("Detaching\n");
	cmd.CommandCode = TSPI_CMD_DetachLUN;
	cmd.AdapterIndex = adapter;
	cmd.Lun = lun;
	tspi_SendCommand(&cmd);
	if (cmd.Error)
		{
		printf("Unable to detach LUN, error %d\n",cmd.Error);
		return;
		}
	}




int main(int argc, char *argv[])
	{
	int NumAdapters = 0;
	unsigned char OurAdapter = 0;
	unsigned char OurLun = 0;

	// Extract adapter/LUN information from the command line
	if (argc >= 2)
		OurAdapter = (unsigned char) atoi(argv[1]);
	if (argc >= 3)
		OurLun = (unsigned char) atoi(argv[2]);

	// Attach to the TSPI manager
	tspi_SendCommand = GetTspiEntryPoint();
	if (tspi_SendCommand)
		{
		TSPI_CMD cmd;
		cmd.CommandCode = TSPI_CMD_AdapterInfo;
		cmd.AdapterIndex = 0;
		tspi_SendCommand(&cmd);
		if (cmd.Error != TSPI_ERR_None)
			{
			printf("Unable to retrieve TSPI adapter information!\n");
			return 3;
			}
		NumAdapters = (int) cmd.Parm[0];
		if (NumAdapters <= 0)
			{
			printf("TSPI manager is installed, but no adapters are available.\n");
			return 3;
			}
		}
	else
		{
		printf("TSPI manager not installed!\n");
		return 3;
		}


	// Run our target code
	RunTarget(OurAdapter,OurLun);

	return 0;
	}
		

	
