/***************************************************************************
 *                                                                         *
 *  Copyright 1995 Symbios Logic Incorporated.  All rights reserved.       *
 *                                                                         *
 * This software was written by Symbios Logic Inc. to develop and test new *
 * products.  Symbios Logic assumes no liability for its use. This software*
 * is released to the public domain to illustrate certain programming      *
 * techniques for testing the 53c8xx chips                                 *
 *                                                                         *
 ***************************************************************************/

/*           Name:  8XX_SCSI.H
 *          Title:  Validation test code 8XX_scsi.c header
 *      $Workfile:  $
 *      $Revision:  $
 *       $Modtime:  $
 *     Programmer: Symbios Logic 
 *  Creation Date:  11/29/95
 *
 *  Version History
 *  ---------------
 *
 *    Date    Who?  Description
 *  --------  ----  -------------------------------------------------------
#BeginRevision
 *  03/06/96  SYM   - Added INITSCSI_RETRIES macro
 *						  - Added IO_Major_Status and IO_Minor_Status macros for
 *						  additional do_rw return values.
 *						  - Added Xfer_status macros for xfer status return values
 *						  in do_rw
 *						  - Added enable_auto_rqs, sense_length, sense_buf,
 *						  IO_Major_status, IO_Minor_status, Xfer_status fields to
 *						  scsirw data structure
 *						  - Added do_StartStopUnit function prototype
#EndRevision
 *
 *  11/29/95  SYM   Initial Revision.
 *
#BeginDescription
 *
 *  This file contains structures and #defines used by the 8XX_SCSI.C source
 *  file.
 *
#EndDescription
 *
 *-------------------------------------------------------------------------
 *
 *  $Header: $
 *
 */

#include "valtypes.h"

/* #defines for scsi operation */

/* Script maint table indirect size */
#define MAINT_TABLE_SIZE 8
/* Script RW table indirect size */
#define RW_TABLE_SIZE 21   
/* Maximum SG entries script can service at one time */
/* DO NOT CHANGE THIS #DEFINE WITHOUT FIRST MODIFYING THE SCRIPT */
/* SUCH THAT IT IS DESIGNED TO HANDLE EXACTLY MAX_SG_ENTRIES */
#define MAX_SG_ENTRIES 16               

/* Number of retries for init_scsi when attemting to clear a check condition */
#define INITSCSI_MAX_RETRIES 4

/* phases of the scsi bus as reported by socl and sbcl */
#define SCSI_MSGO_PHASE 6
#define SCSI_MSGI_PHASE 7
#define SCSI_STATUS_PHASE 3
#define SCSI_CMD_PHASE 2
#define SCSI_DATAI_PHASE  1
#define SCSI_DATAO_PHASE 0

/* return codes */
/* Returned by any function in 8XX_SCSI.C which completes successfully */
#define SCSI_GOOD 0x00
/* Returned by any of the do_ functions where there was a 
	selection time-out */
#define SCSI_SELECTTO 0x01
/* Returned by do_syncneg or do_wideneg to indicate that the device has
	rejected the sync or wide negotiation attempt */
#define SCSI_MSGREJECT 0x02
/* Returned by any of the do_ functions where a SCSI command failed but the 
	SCSI bus is no longer in use */
#define SCSI_COMMANDFAILED 0x81
/* Returned by any of the do_ functions where a SCSI command failed and the 
	target that was being accessed is still connected to the bus. */
#define SCSI_CON_COMMANDFAILED 0x82
/* Returned by any of the do_ maintenance functions (do_tur, do_rqsense, 
	do_inquiry, do_rdcap, do_syncneg, and do_wideneg) where an illegal 
	interrupting condition occurred */
#define SCSI_ILLEGAL_INT 0x83
/* Returned by any of the do_ maintenance functions where there was no 
	interrupt in the specified time-out period */
#define SCSI_POLLTO 0x84
/* Returned by any of the do_ maintenance functions where there was an 
	unknown error condition. */
#define SCSI_UNKNOWN 0x85

/* IO Major status bytes */
/* Standard command complete interrupt */
#define SCSI_COMMAND_COMPLETE 0x00
/* Illegal SCSI interrupt status, the minor status will have the
	specific interrupt */
#define SCSI_ILLEGAL_SCSI_INT 0x01
/* Illegal Script interrupt status, the minor status will have the
	specific interrupt vector */
#define SCSI_ILLEGAL_SCRIPT_INT 0x02
/* Illegal dma interrupt status,  the minor status will have the
	specific interrupt vector */
#define SCSI_ILLEGAL_DMA_INT 0x03
/* Unknown Illegal interrupt, no minor status */
#define SCSI_UNKNOWN_ILLEGAL_INT 0x04
/* IO returned Check condition status */
#define SCSI_CHECK_CONDITION 0x05
/* No interrupt was recieved in the timeout period, no minor status */
#define SCSI_NO_INTERRUPT 0x06

/* IO Minor status bytes */
/* Function complete int */
#define SCSI_GOT_CMP_INT 0x01
/* Selected Int */
#define SCSI_GOT_SEL_INT 0x02
/* Reselected Int */
#define SCSI_GOT_RSL_INT 0x03
/* SCSI Gross error int */
#define SCSI_GOT_SGE_INT 0x04
/* Unexpected disconnect int */
#define SCSI_GOT_UDC_INT 0x05
/* SCSI Reset int */
#define SCSI_GOT_RST_INT 0x06
/* SCSI Parity error int */
#define SCSI_GOT_PAR_INT 0x07
/* SCSI General purpose timeout int */
#define SCSI_GOT_GEN_INT 0x08
/* Handshake to Handshake timeout int */
#define SCSI_GOT_HTH_INT 0x09
/* Script interrupt indicating that a tag message was expected but not */
/* received */
#define SCSI_ILLEGAL_TAG_MESSAGE 0x0a
/* Script interrupt indicating that a message byte was sent that cannot */
/* be handled by the script */
#define SCSI_UNEXPECTED_MESSAGE  0x0b
/* Script interrupt indicating that there was a phase on the bus the */
/* the bit bucket routine cannot handle */
#define SCSI_BITBUCKET_FAILED 0x0c
/* DMA Master data parity error */
#define SCSI_GOT_MDPE_INT 0x0d
/* DMA bus fault int */
#define SCSI_GOT_BF_INT 0x0e
/* DMA aborted int */
#define SCSI_GOT_ABRT_INT 0x0f
/* DMA single step int */
#define SCSI_GOT_SSI_INT 0x10
/* DMA illegal instruction int */
#define SCSI_GOT_IID_INT 0x11
/* Used for check condition status when the sense data is valid */
#define SCSI_VALID_SENSE_DATA 0x12
/* Used for check condition status when the sense data is invalid */
#define SCSI_INVALID_SENSE_DATA 0x13
/* Indicatates that no status for this condition has been assigned */
/* can be used for major or minor statuses */
#define SCSI_NO_STATUS 0xff

/* Transfer status bytes */
/* indicates that all data was transferred */
#define SCSI_TRANSFER_COMPLETE 0x00
/* indicates that not all data was transferred */
#define SCSI_INCOMPLETE_TRANSFER 0x01

/* enumerations */

/* maint_table_offsets - enumerated list for use with the core maintenance
	table indirect script.  Should not be used by the test function. */

enum maint_table_offsets {
	MAINT_SCSI_ID = 0,
	MAINT_MSGO_BUF,
	MAINT_CMD_BUF,
	MAINT_STAT_BUF,
	MAINT_MSGI_BUF,
	MAINT_EMSGI_BUF,
	MAINT_DATA_BUF
};

/* rw_table_offsets - enumerated list for use with the core read/write table 
	indirect script.  Should not be used by the test function. */

enum rw_table_offsets {
		  RW_SCSI_ID = 0,
		  RW_ID_BUF,
		  RW_CMD_BUF,
		  RW_STAT_BUF,
		  RW_MSGI_BUF,
		  RW_DATA_BUF1,
		  RW_DATA_BUF2,
		  RW_DATA_BUF3,
		  RW_DATA_BUF4,
		  RW_DATA_BUF5,
		  RW_DATA_BUF6,
		  RW_DATA_BUF7,
		  RW_DATA_BUF8,
		  RW_DATA_BUF9,
		  RW_DATA_BUF10,
		  RW_DATA_BUF11,
		  RW_DATA_BUF12,
		  RW_DATA_BUF13,
		  RW_DATA_BUF14,
		  RW_DATA_BUF15,
		  RW_DATA_BUF16
};

/* scsimaint - structure required by the maintenance scsi functions 
	(do_tur, do_rqsense, do_inquiry, do_readcap, do_syncneq, and 
	do_wideneg) All items marked input must be filled in depending 
	on the function called. */

typedef struct {
	VULONG sel_info;        /* input/output */
	VULONG data_length;     /* input/output */
	VUBYTE msgo_buf[6];     /* input */
	VUBYTE cmd_buf[12];     /* output */
	VUBYTE stat_buf;        /* output */
	VUBYTE msgi_buf;        /* output */
	VUBYTE emsgi_buf[4];    /* output */
	VUBYTE *data_buf;       /* input/output */
} scsimaint;

/* field descriptions for scsimaint:
	sel_info - A select table indirect entry which contains the 
		appropriate information for the device being selected.  This is 
		required to be filled in for all maintenance functions.  This 
		field is modified by the do_syncneg and do_wideneg to reflect 
		the proper sxfer and scntl3 values for the respective 
		negotiation that took place.
	data_length - This field must be filled in to indicate the amount of 
		data desire for the do_rqsense and do_inquiry functions.  
		Upon return from those functions this field contains the actual 
		amount of data that was sent from the target during the 
		command.
	msgo_buf - Array item 0 of this field must be filled in with the 
		appropriate identify message (0x8X or 0xCX).  The remaining 
		array items can be filled in with a tag message or in the case 
		of do_syncneg or do_wideneg must be filled in with the 
		appropriate negotiation messages.  Tag messages are not 
		allowed when doing sync or wide negotiation.
	cmd_buf - This array field is filled in by the maintenance 
		function.  It is not necessary to fill in this field.  Upon return 
		from the function the actual scsi command used will be in this 
		array.
	stat_buf - This field is filled in by the maintenance function.  
		Upon return from the function the status byte received from 
		the target will be located here.
	msgi_buf - This field is filled in by the maintenance function.  
		Upon return from the function the msg in byte from the target 
		at the completion of the command will be stored here.
	emsgi_buf - This array field is filled in by the do_syncneg and 
		do_wideneg with the return negotiation message.  It is not 
		necessary to use this information to program the chip for wide 
		or sync operation as the do_wideneg and do_syncneg 
		functions perform this operation and place the appropriate 
		values in the sel_info field above.
	data_buf - This pointer must be assigned to a valid data buffer 
		when executing a do_rqsense, do_inquiry, or do_rdcap 
		function.  The buffer must be large enough to hold the 
		maximum amount of data requested (i.e. it must be at least 
		data_length in size or 10 bytes in the case of a read 
		capacity)
*/

/* scsirw - structure required by the read/write scsi functions (do_read6, 
	do_read10, do_write6, do_write10, and do_rw) All items marked 
	input must be filled in. */

typedef struct {
	ti_entry *sg_entry;     /* input */
	VULONG sel_info;        /* input */
	VULONG lba;             /* input */
	VULONG blk_count;       /* input */
	VINT id_count;          /* input */
	VINT sg_count;          /* input */
	VUBYTE enable_auto_rqs;	/* input */
	VUBYTE id_buf[3];       /* input */
	VUBYTE cmd_buf[12];     /* output */
	VUBYTE sense_length;		/* output */
	VUBYTE sense_buf[18];	/* output */
	VUBYTE IO_major_status;	/* output */
	VUBYTE IO_minor_status;	/* output */
	VUBYTE Xfer_status;		/* output */
	VUBYTE stat_buf;        /* output */
	VUBYTE msgi_buf;        /* output */
	VUBYTE resel_msg_buf[3];        /* output */
} scsirw;

/* field descripttions for scsirw:
	sg_entry - This is a pointer to a scatter gather list.  This pointer 
		must be assigned to a valid scatter gather list.  Note that even 
		if scatter gather is not used (i.e. only on data Table indirect 
		entry) this pointer must be assigned to that single entry as this
		is the sole mechanism for passing byte count and address 
		information to the function.
	sel_info - This is the standard selection table indirect entry.  It 
		must be filled in with the appropriate sync, wide and 
		destination id information in the correct format.  See the 
		ti_entry data structure description for the Select Table Indirect 
		entry format.
	lba - This is the starting Logical Block Address on the for the
		read or write transfer.  This value should not exceed the 
		maximum LBA as reported by the target device.
	blk_count - This is the block count for the read or write transfer.  
		This value plus the LBA must also not exceed the maximum 
		LBA as reported by the target device.
	id_count - This is the number of initial message bytes that should 
		be sent to the target device.  It will generally be 1 or 3, 1 
		indicating only an identify message should be sent and 3 
		indicated an identify plus tag message should be sent.
	sg_count - This is the number of scatter gather entries for this IO.
		Note that non scatter gather IO's should initialize this value to
		one as all IOs are considered to be scatter gather, non-scatter
		gather IOs will just have an sg_count of 1.
	enable_auto_rqs - Enable Auto Request Sense.  If the IO completes with
		a check condition the a request sense will be issued automatically
		if this flag is set.  Otherwise no request sense will be issued.
	id_buf - This field should be filled in with the identify message
		for the IO and optionally a tag message for tagged command
		queuing.
	cmd_buf - This field hold the actual command of the read or
		write.  It is not necessary to fill in this field except for the
		do_rw function.  Upon return from the read or write function
		this command will contain the command sent to the target
		device.  If the do_rw function is being called this field must be
		filled in with a read or write type command that is compatible
		with the do_rw function.
	stat_buf - Upon successful return this field will contain the status
		byte from the target at the end IO.
	sense_length - If the IO completed with a check condition then this
		field will contain the number of request sense byte in the
		sense buffer
	sense_buf - If the IO completes with a check condition then this buffer
		will contain the sense data from an auto request sense.
	msgi_buf - Upon successful return this field will contain message
		in byte from the target at the end of the IO.
	IO_major_status - This contains the major status of the IO on return.  This
		indicatates the general cause of a failure if there is in fact a failure.
		See the above list of status values for all possible return values.
	IO_minor_status - This contains the minor status of the IO on return.
		This indicates the specific cause of a failure if thare is in fact a failure.
		See the above list of status values for all possible return values.
	Xfer_status - This status byte indicates if the transfer completed or not.
		See the above list for Xfer_status values.
	resel_msg_buf - This field is used to hold the reselection identify
		message and option tag from the target device after a
		reselection.  Upon successful return of a disconnecting IO this
		field will contain the last identify and tag message the target
		device sent.  If the IO is not disconnecting this field is
		undefined.

*/


/* scsidev_record - This structure is returned as a linked list by the 
	init_scsi routine.  It contains information about each device that 
	was found on the SCSI bus. */

typedef struct _scsidev_record  {
	VUBYTE id_type;
	VUBYTE flags;                    /* 0-sync,1-wide,2-cmdq,3-remove */
	VULONG sel_info;
	VULONG maxlba;
	VULONG blksize;
	struct _scsidev_record *nxt_dev;
} scsidev_record;

/* field descriptions for scsidev_record:
	id_type - This field contains the target id of the device found and 
		the type of the device as report in the inquiry data.
	flags - This field contains certain flags indicating certain 
		capabilities of the device.  Bit 0 indicates sync capability, bit 1 
		indicates wide capability, bit 2 indicates tag command 
		queuing capability, and bit 3 indicates if the medium is 
		removable.  Note that these flags only indicate capability, they 
		do not indicate the current state of the device.
	sel_info - This is the standard selection table indirect entry.  It 
		must be filled in with the appropriate sync, wide and 
		destination id information in the correct format.  See the 
		ti_entry data structure description for the Select Table Indirect 
		entry format.
	maxlba - This is the Maximum Logical Block address allowed by 
		the device as reported by the Read Capacity command.
	blksize - This is the Block size of the device in bytes as reported 
		in the Read Capacity command.
	nxt_dev - This is a pointer to the next device in the linked list.

*/

/* function prototypes */
extern VINT init_gpscripts(void);
extern scsidev_record *init_scsi(pcidev_record *PCIDevice, VINT do_sync, VINT do_wide);
extern VINT do_TUR(pcidev_record *PCIDevice, scsimaint *maint);
extern VINT do_RqSense(pcidev_record *PCIDevice, scsimaint *maint);
extern VINT do_Inquiry(pcidev_record *PCIDevice, scsimaint *maint);
extern VINT do_RdCap(pcidev_record *PCIDevice, scsimaint *maint);
extern VINT do_StartStopUnit(pcidev_record *PCIDevice, scsimaint *maint);
extern VINT do_SyncNeg(pcidev_record *PCIDevice, scsimaint *maint);
extern VINT do_WideNeg(pcidev_record *PCIDevice, scsimaint *maint);
extern VINT do_Read6(pcidev_record *PCIDevice, scsirw *rw);
extern VINT do_Read10(pcidev_record *PCIDevice, scsirw *rw);
extern VINT do_Write6(pcidev_record *PCIDevice, scsirw *rw);
extern VINT do_Write10(pcidev_record *PCIDevice, scsirw *rw);

