/* pci.h

	Standard PCI functions header
	Author: Andy Brown

	#include this file if you are linking the pci.c code into your
	program

*/

#define		PCI_BIOS_INT				0x1A
#define		PCI_FUNCTION_ID			0xB1
#define		PCI_SYM_VENDOR_ID			0x1000

#define		PCI_BIOS_PRESENT			0x01
#define		PCI_FIND_DEVICE			0x02
#define		PCI_READ_CONFIG_DWORD	0x0A
#define		PCI_WRITE_CONFIG_DWORD	0x0D

#define		PCI_DEVICE_FOUND			0x01
#define		PCI_NO_DEVICE_FOUND		0x00

#define		PCI_BIOS_REV_1X			0x10
#define		PCI_BIOS_REV_2X			0x20
#define		PCI_UNKNOWN_BIOS			0xFF
#define		PCI_NO_BIOS					0x00

/* PCI configuration regs */
#define		PCI_CONFIG_REG_VENID		0x00
#define		PCI_CONFIG_REG_DEVID 	0x00
#define		PCI_CONFIG_REG_CMD		0x04
#define		PCI_CONFIG_REG_STAT		0x04
#define 		PCI_CONFIG_REG_REVID		0x08
#define		PCI_CONFIG_REG_CLASS		0x08
#define		PCI_CONFIG_REG_CLS		0x0C
#define		PCI_CONFIG_REG_LAT		0x0C
#define		PCI_CONFIG_REG_HDRT		0x0C
#define		PCI_CONFIG_REG_BIST		0x0C
#define		PCI_CONFIG_REG_CIS		0x28
#define		PCI_CONFIG_REG_SUBV		0x2C
#define		PCI_CONFIG_REG_SUBID		0x2C
#define		PCI_CONFIG_REG_ROM		0x30
#define		PCI_CONFIG_REG_INTL		0x3C
#define		PCI_CONFIG_REG_INTP		0x3C
#define		PCI_CONFIG_REG_MGNT		0x3C
#define		PCI_CONFIG_REG_MLAT		0x3C


/* Structures returned and required by functions */
/* type device_record returned by FindSYMDevice */
/* type device record required by GetConfigRegister and SetConfigRegister */
typedef struct {
			unsigned int  bus_num;
			unsigned int  device_num;
			unsigned int  function;
			unsigned int  device_id;
			unsigned int  device_index;
			unsigned char rev_id;
			unsigned char CLS;
			unsigned char latency;
			unsigned int  command;
			unsigned long base_addr1;
			unsigned long base_addr2;
			unsigned long base_addr3;
			unsigned long base_addr4;
			unsigned long base_addr5;
			unsigned long base_addr6;
			unsigned long CIS;
			unsigned int  sub_id;
			unsigned int  sub_ven_id;
			unsigned long ROM_base_addr;
			unsigned char intl;
			unsigned char intp;

} device_record;

/* structure returned by PCI_GetPCIBIOSVersion */
typedef struct {
			unsigned char access_mech;
			unsigned int interface_lvl;
			unsigned char last_bus;
} pci_bios_info;

/* function prototypes */
extern int PCI_FindSYMDevice(device_record *Device);
extern void PCI_SetConfigRegister(device_record *Device, unsigned int offset, unsigned long value);
extern unsigned long PCI_GetConfigRegister(device_record *Device, unsigned int offset);
extern unsigned int  PCI_GetPCIBIOSVersion(pci_bios_info *pcibios);

